# -*- coding: utf8 -*-
import logging
import subprocess

logger = logging.getLogger(__name__)


def detect_test_runners(files):
    if 'Makefile' in files:
        return ['make test']
    if 'tox.ini' in files:
        return ['tox', 'flake8']
    if 'setup.py' in files:
        return ['python setup.py test', 'flake8']
    if 'manage.py' in files:
        return ['python manage.py test', 'flake8']
    if 'package.json' in files:
        return ['npm install', 'npm test']
    if 'build.sbt' in files:
        return ['sbt test']
    if 'Cargo.toml' in files:
        return ['cargo test']
    if '_config.yml' in files:
        return ['jekyll build']
    return []


class ProcessResult(object):

    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)


def local_run(command, directory=None):
    command = '{} 2>&1'.format(command)
    if directory:
        command = 'cd {} && {}'.format(directory, command)

    logger.debug('Running command: {}'.format(command))
    result = ProcessResult(command=command)
    print(command)

    process = subprocess.Popen(
        ['/bin/bash', '-i', '-c'] + command.split(' '),
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        close_fds=True
    )

    (stdout, stderr) = process.communicate()
    result.out = stdout.decode('utf-8').strip() if stdout else ''
    result.err = stderr.decode('utf-8').strip() if stderr else ''
    result.return_code = process.returncode
    result.succeeded = result.return_code == 0
    logger.debug('Result: {}'.format(result.__dict__))
    return result


class CachedProperty(object):
    def __init__(self, func, name=None):
        self.func = func
        self.__doc__ = getattr(func, '__doc__')
        self.name = name or func.__name__

    def __get__(self, instance, type=None):
        if instance is None:
            return self
        res = instance.__dict__[self.name] = self.func(instance)
        return res


cached_property = CachedProperty



# -*- coding: utf-8 -*-
import logging
import os

import yaml

logger = logging.getLogger(__name__)

PATHS = [
    '/etc/frigg/config.yaml',
    '/etc/frigg/config.yml',
    os.path.expanduser('~/.frigg/config.yaml'),
    os.path.expanduser('~/.frigg/worker.yml'),
]


def config(key):
    settings = yaml.load(open(os.path.join(os.path.dirname(__file__), 'defaults.yml')))
    for path in PATHS:
        try:
            settings.update(yaml.load(open(path)))
        except IOError:
            pass
    return settings.get(key, None)


def redis_client():
    try:
        import redis
        return redis.Redis(
            host=config('REDIS')['host'],
            port=config('REDIS')['port'],
            db=config('REDIS')['db'],
            password=config('REDIS')['password']
        )
    except ImportError:
        logger.warning('You need to install the redis python package in order to'
                       ' use functionality that relies on redis.')


try:
    from raven import Client
    sentry = Client(config('SENTRY_DSN'))  # noqa
except ImportError:
    logger.debug('Raven is not installed.')
    sentry = None



# -*- coding: utf8 -*-
import logging
from os import listdir
from os.path import exists, isfile, join

import yaml

from .helpers import detect_test_runners

logger = logging.getLogger(__name__)


def build_tasks(directory):
    try:
        files = [f for f in listdir(directory) if isfile(join(directory, f))]
    except OSError as e:
        files = []
        logger.error('Could not read files in path {}: \n {}'.format(directory, e))
    return detect_test_runners(files)


def load_settings_file(path):
    with open(path) as f:
        return yaml.load(f)


def get_path_of_settings_file(directory):
    if exists(join(directory, '.frigg.yml')):
        return join(directory, '.frigg.yml')
    elif exists(join(directory, '.frigg.yaml')):
        return join(directory, '.frigg.yaml')


def build_settings(directory):
    path = get_path_of_settings_file(directory)

    settings = {
        'webhooks': [],
    }

    if path is not None:
        settings.update(load_settings_file(path))
    else:
        settings['tasks'] = build_tasks(directory)

    if len(settings['tasks']) == 0:
        raise RuntimeError('No tasks found')

    return settings



# -*- coding: utf8 -*-
import json
import logging
import os

from frigg_coverage import parse_coverage

from frigg import api
from frigg.config import config, sentry
from frigg.helpers import cached_property, local_run
from frigg.projects import build_settings

logger = logging.getLogger(__name__)


class Result(object):
    log = ''
    return_code = None
    succeeded = None
    task = None

    def __init__(self, task, result=None, error=None):
        self.task = task
        if result:
            self.succeeded = result.succeeded
            self.return_code = result.return_code
            self.log = result.out
        if error:
            self.log = error or result.err

    @classmethod
    def serialize(cls, obj):
        if isinstance(obj, dict):
            return obj
        return obj.__dict__


class Build(object):
    id = ''
    results = []
    cloned = False
    branch = 'master'
    sha = None
    clone_url = None
    name = None
    owner = None
    pull_request_id = None
    coverage = None

    def __init__(self, build_id, obj):
        self.__dict__.update(obj)
        self.id = build_id
        self.results = []

    @property
    def working_directory(self):
        return os.path.join(config('TMP_DIR'), str(self.id))

    @property
    def succeeded(self):
        for result in self.results:
            if result.succeeded is False:
                return False
        return True

    @cached_property
    def settings(self):
        return build_settings(self.working_directory)

    def run_tests(self):
        self.delete_working_dir()
        if not self.clone_repo():
            return self.error('git clone', 'Access denied')

        try:
            for task in self.settings['tasks']:
                self.run_task(task)

            if self.settings['coverage']:
                self.coverage = parse_coverage(
                    os.path.join(self.working_directory, self.settings['coverage']['path']),
                    self.settings['coverage']['parser']
                )

        except Exception as e:
            self.error('', e)
            sentry.captureException()
            logger.error('Build nr. %s failed\n%s' % (self.id, e.message))
        finally:
            self.delete_working_dir()
            self.report_run()
            logger.info("Run of build %s finished." % self.id)

    def clone_repo(self, depth=1):
        command_options = {
            'depth': depth,
            'url': self.clone_url,
            'pwd': self.working_directory,
            'branch': self.branch,
            'pr_id': self.pull_request_id
        }
        if self.pull_request_id is None:
            clone = local_run("git clone --depth=%(depth)s --branch=%(branch)s "
                              "%(url)s %(pwd)s" % command_options)
        else:
            clone = local_run(
                ("git clone --depth=%(depth)s %(url)s %(pwd)s && cd %(pwd)s "
                 "&& git fetch origin pull/%(pr_id)s/head:pull-%(pr_id)s "
                 "&& git checkout pull-%(pr_id)s") % command_options
            )
        if not clone.succeeded:
            message = "Access denied to %s/%s" % (self.owner, self.name)
            logger.error(message)
        return clone.succeeded

    def run_task(self, task_command):
        run_result = local_run(task_command, self.working_directory)
        self.results.append(Result(task_command, run_result))

    def delete_working_dir(self):
        if os.path.exists(self.working_directory):
            local_run("rm -rf %s" % self.working_directory)

    def error(self, task, message):
        self.results.append(Result(task, error=message))
        self.errored = True

    def report_run(self):
        return api.report_run(self.id, json.dumps(self, default=Build.serializer)).status_code

    @classmethod
    def serializer(cls, obj):
        out = obj.__dict__
        if isinstance(obj, Build):
            out['results'] = [Result.serialize(r) for r in obj.results]
            try:
                out['settings'] = obj.settings
            except RuntimeError:
                pass
        return out



# -*- coding: utf8 -*-
import logging

import requests

from frigg.config import config

logger = logging.getLogger(__name__)


def report_run(build_id, build):
    response = requests.post(
        config('HQ_REPORT_URL'),
        data=build,
        headers={
            'content-type': 'application/json',
            'FRIGG_WORKER_TOKEN': config('TOKEN')
        }
    )
    logger.info('Reported build to hq, hq response status-code: %s, data:\n%s' % (
        response.status_code,
        build
    ))
    if response.status_code != 200:
        with open('build-%s-hq-response.html' % build_id, 'w') as f:
            f.write(response.text)
    return response



# -*- encoding: utf8 -*-
from setuptools import setup, find_packages

setup(
    name='frigg-common',
    version='0.1.1',
    description='Utils for frigg apps',
    author='The frigg team',
    author_email='hi@frigg.io',
    license='MIT',
    url='https://github.com/frigg/frigg-common',
    packages=find_packages(exclude=['tests', 'tests.*']),
    include_package_data=True,
    install_requires=[
        'frigg-coverage',
        'pyyaml',
        'requests',
    ],
    classifiers=[
        'Programming Language :: Python',
    ]
)
